/************************************************************************/
/*                                                                      */
/*  MFLSYS header file                                                  */
/*                                                                      */
/*  Defines and function prototypes for printer access, serial          */
/*  communications, gameport access, and miscellaneous system functions.*/
/*                                                                      */
/*  Copyright 1989 by Robert B. Stout dba MicroFirm                     */
/*  All rights reserved                                                 */
/*                                                                      */
/*  Copyright 1986, 1987 by S.E. Margison                               */
/*                                                                      */
/*  Compiled by QC 2.0 for use with MSC 5.1 or QC 2.0 or later.         */
/*                                                                      */
/************************************************************************/

#ifndef MFL_SYS_H
#define MFL_SYS_H

#include <mfldefs.h>

#ifndef _Cdecl_
 #ifdef NO_EXT_KEYS
  #define _Cdecl_
 #else
  #define _Cdecl_ cdecl
 #endif
#endif

/************************************************************************/
/*                                                                      */
/*  Encrypt/decrypt buffer data                                         */
/*                                                                      */
/************************************************************************/

extern char *cryptext;          /* The actual encryption/decryption key */
extern int crypt_ptr;           /* Circular pointer to elements of key  */
extern int crypt_length;        /* Set by caller to strlen(cryptext)    */

void    _Cdecl_ crypt(char *);
LOGICAL _Cdecl_ cryptqual(char *, int);

/************************************************************************/
/*                                                                      */
/*  Serial communications functions                                     */
/*                                                                      */
/************************************************************************/

void    _Cdecl_ setport(int, int),
        _Cdecl_ setdtr(int, int),
        _Cdecl_ setrts(int, int),
        _Cdecl_ writechar(int, int);

int     _Cdecl_ readchar(int),
        _Cdecl_ ready_recv(int),
        _Cdecl_ ready_xmt(int),
        _Cdecl_ chkdsr(int),
        _Cdecl_ chkdcd(int),
        _Cdecl_ chkcts(int),
        _Cdecl_ chkring(int),
        _Cdecl_ config_port(unsigned, int, int, int, int);

/************************************************************************/
/*                                                                      */
/*  Printer functions                                                   */
/*                                                                      */
/************************************************************************/

void    _Cdecl_ pr_putline(char *),
        _Cdecl_ pr_puts(char *),
        _Cdecl_ pr_set(int),
        _Cdecl_ pr_nl(void),
        _Cdecl_ pr_eject(void),
        _Cdecl_ pr_carret(void);

int     _Cdecl_ blpr(char, int),
        _Cdecl_ blprstat(int),
        _Cdecl_ pr_putc(char),
        _Cdecl_ PrtSc(void),
        _Cdecl_ getPrtSc_stat(void);

/************************************************************************/
/*                                                                      */
/*  Gameport functions                                                  */
/*                                                                      */
/************************************************************************/

void            _Cdecl_ init_game(unsigned char),
                _Cdecl_ clear_game(void),
                _Cdecl_ debounce(void);

unsigned char   _Cdecl_ get_press(void);

/************************************************************************/
/*                                                                      */
/*  Error handling functions                                            */
/*                                                                      */
/************************************************************************/

void    _Cdecl_ aabort(int),
        _Cdecl_ error(char *);

int     _Cdecl_ ctlbrk();

/************************************************************************/
/*                                                                      */
/*  Miscellaneous system functions                                      */
/*                                                                      */
/************************************************************************/

void    _Cdecl_ get_code_adr(void *, unsigned int *, unsigned int *),
        _Cdecl_ get_data_adr(void *, unsigned int *, unsigned int *);

int     _Cdecl_ bdosx(int, void *, char),
	_Cdecl_ _getdi(int),
        _Cdecl_ ibmtype(void);

LOGICAL _Cdecl_ ansisys(void);

#ifndef COLD
 #define COLD 0
#endif

#ifndef WARM
 #define WARM 1
#endif

void    _Cdecl_ reboot(int);

#endif /* MFL_SYS_H */
